// my-plugin.js
// 使用 `unist-util-visit` 来遍历树
import { SKIP, visit } from "unist-util-visit";
import { unified } from "unified";
import parse from "remark-parse";
import remarkStringify from "remark-stringify";

// const pos = [500, 750];
// const [start, end] = pos;

function checkMarked(str: string) {
  if (str.startsWith("<mark>")) {
    // 说明已经标记了 skip掉
    return true;
  }
  return false;
}

const myPlugin = (options) => {
  console.log("🚀 ~ myPlugin ~ options:", options);
  const { pos } = options;
  const [start, end] = pos;
  return (tree) => {
    // 遍历树并转换高亮
    if (tree) {
      visit(tree, "text", (node) => {
        const nodePositionStart = node.position.start;
        const nodePositionEnd = node.position.end;

        // 标记 开始 <mark>
        if (
          nodePositionStart.offset <= start &&
          nodePositionEnd.offset >= start
        ) {
          node.value = `<mark>${node.value}</mark>`;
          // 不转换此node的子节点
          SKIP;
        }

        // 如果打过标记的 跳过
        // 标记点必须大于 start
        if (
          end >= nodePositionStart.offset &&
          nodePositionStart.offset >= start
        ) {
          if (!checkMarked(node.value))
            node.value = `<mark>${node.value}</mark>`;
        }
        console.log("🚀 ~ visit ~ node:", node);
      });
    }
  };
};

// 1. 计算在文档里面的位置？？
// 2. 位置准确，高亮可能不准确

const processor = ({ pos }: { pos: number[] }) =>
  unified()
    // parse 将 markdown string 转成 ast tree
    .use(parse)
    // 添加插件 来处理 ast tree
    .use(myPlugin, { pos })
    // 将 ast tree 转成 markdown string
    .use(remarkStringify);

// processor.process(input, (err, file) => {
//   if (err) throw err;
//   let result = String(file);
//   // 去掉转义符等
//   result = result.replace(/[\'\"\\\/\b]/g, "");
//   console.log("🚀 ~ processor.process ~ result:", result);
// });

export default processor;
